@@ -25,16 +25,18 @@ from itertools import product
 from multiprocessing import Pool
 import operator
 import sys
-import re
-from collections import namedtuple
-
-import numpy as np
+from collections import namedtuple, defaultdict
 import pandas as pd
+import time
 
 import psamm.bayesian_util as util
 from psamm.formula import Formula
+from psamm.expression.boolean import Expression
 from functools import reduce
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 CompoundEntry = namedtuple(
     'CompoundEntry',
@@ -58,9 +60,12 @@ class MappingModel(object):
         self._read_reactions(model.reactions)
 
         self._genes = set()
+        self._has_gene = list()
         for r in itervalues(self._reactions):
             if r.genes is not None:
-                self._genes.update(r.genes)
+                self._has_gene.append(r.id)
+                e = Expression(r.genes)
+                self._genes.update([g.symbol for g in e.variables])
 
     def _read_compounds(self, it):
         self._compounds = {}
@@ -88,16 +93,6 @@ class MappingModel(object):
         self._reactions = {}
         for reaction in it:
             genes = getattr(reaction, 'genes', None)
-            if genes is not None:
-                # Delete operators in string
-                genes = re.sub(r'[,()\']*', '', genes)
-                genes = re.sub(r'\band\b', '', genes)
-                genes = re.sub(r'\bor\b', '', genes)
-                genes = re.split(r'\s+', genes)  # Transfer string to list
-                genes = frozenset(genes)
-            else:
-                genes = None
-
             reaction_id = reaction.id
             equation = getattr(reaction, 'equation', None)
             entry = ReactionEntry(
@@ -129,19 +124,24 @@ class MappingModel(object):
         print('- Compounds: {}'.format(len(self.compounds)))
         print('- Reactions: {}'.format(len(self.reactions)))
         print('- Genes: {}'.format(len(self.genes)))
+        print('- Reactions with gene association: {}'.format(
+            len(self._has_gene)))
 
     def check_reaction_compounds(self):
         """Check that reaction compounds are defined in the model"""
         print('Checking model: {}'.format(self.name))
         consistent = True
+        undefined_cpds = []
         for reaction in itervalues(self.reactions):
             if reaction.equation is not None:
                 for compound, value in reaction.equation.compounds:
                     if compound.name not in self.compounds:
-                        print((
-                            '{} in reaction {} '
-                            'is not in compound list').format(
-                                compound.name, reaction.id))
+                        undefined_cpds.append(compound.name)
+                        logger.error((
+                            '{} in reaction {} is not defined in compound '
+                            'list (such as compounds.yaml), please define it '
+                            'in compound list before running modelmapping'
+                        ).format(compound.name, reaction.id))
                         consistent = False
         return consistent
 
@@ -218,16 +218,22 @@ class BayesianReactionPredictor(object):
         gene: whether to compare the gene association.
         compartment_map: dictionary mapping compartment id in the query model
                          to the id in the target model.
+        gene_map: dictionary mapping gene id in the query model to the id in
+                  the target model.
     """
 
     def __init__(self, model1, model2, cpd_pred, nproc=1,
-                 outpath='.', log=False, gene=False, compartment_map={}):
+                 outpath='.', log=False, gene=False,
+                 compartment_map={}, gene_map={}):
         self._model1 = model1
         self._model2 = model2
+        self._parse_cpd_pred(cpd_pred)
         self._column_list = ['p', 'p_id', 'p_name', 'p_equation', 'p_genes']
+        gene_map = self._reversible_map(gene_map)
         self._reaction_map_p = map_model_reactions(
-            self._model1, self._model2, cpd_pred, nproc, outpath,
-            log=log, gene=gene, compartment_map=compartment_map)
+            self._model1, self._model2, self._cpd_map, self._cpd_score, nproc,
+            outpath, log=log, gene=gene, compartment_map=compartment_map,
+            gene_map=gene_map)
 
     @property
     def model1(self):
@@ -240,6 +246,20 @@ class BayesianReactionPredictor(object):
     def map(self, r1, r2):
         return self._reaction_map_p[0][r1, r2]
 
+    def _reversible_map(self, genemap):
+        newmap = {}
+        for k, v in genemap.items():
+            newmap[k] = v
+            newmap[v] = k
+        return newmap
+
+    def _parse_cpd_pred(self, cpd_pred):
+        self._cpd_map = defaultdict(set)
+        self._cpd_score = dict()
+        for pair, score in cpd_pred.items():
+            self._cpd_map[pair[0]].add(pair[1])
+            self._cpd_score[pair[0]] = score
+
     def get_raw_map(self):
         """Return pandas.DataFrame style of raw mapping table."""
         reaction_result = pd.DataFrame({
@@ -263,12 +283,12 @@ class BayesianReactionPredictor(object):
 
 def compound_id_likelihood(c1, c2, compound_prior, compound_id_marg):
     if util.id_equals(c1.id, c2.id):
-        p_match = 0.65
+        p_match = 0.4
         p_marg = compound_id_marg
         p_no_match = max(
             0, (p_marg - p_match * compound_prior) / (1.0 - compound_prior))
     else:
-        p_match = 0.35
+        p_match = 0.6
         p_marg = 1.0 - compound_id_marg
         p_no_match = max(
             0, (p_marg - p_match * compound_prior) / (1.0 - compound_prior))
@@ -303,14 +323,14 @@ def compound_charge_likelihood(
         p_match = 0.9
         p_no_match = max(
             0,
-            ((compound_charge_equal_marg - p_match * compound_prior) /
-             (1.0 - compound_prior)))
+            ((compound_charge_equal_marg - p_match * compound_prior)
+             / (1.0 - compound_prior)))
     else:
         p_match = 0.1
         p_no_match = max(
             0,
-            ((compound_charge_not_equal_marg - p_match * compound_prior) /
-             (1.0 - compound_prior)))
+            ((compound_charge_not_equal_marg - p_match * compound_prior)
+             / (1.0 - compound_prior)))
 
     return p_match, p_no_match
 
@@ -327,14 +347,14 @@ def compound_formula_likelihood(
         p_match = 0.9
         p_no_match = max(
             0,
-            ((compound_formula_equal_marg - p_match * compound_prior) /
-             (1.0 - compound_prior)))
+            ((compound_formula_equal_marg - p_match * compound_prior)
+             / (1.0 - compound_prior)))
     else:
         p_match = 0.1
         p_no_match = max(
             0,
-            ((compound_formula_not_equal_marg - p_match * compound_prior) /
-             (1.0 - compound_prior)))
+            ((compound_formula_not_equal_marg - p_match * compound_prior)
+             / (1.0 - compound_prior)))
 
     return p_match, p_no_match
 
@@ -351,14 +371,14 @@ def compound_kegg_likelihood(
         p_match = 0.65
         p_no_match = max(
             0,
-            ((compound_kegg_equal_marg - p_match * compound_prior) /
-             (1.0 - compound_prior)))
+            ((compound_kegg_equal_marg - p_match * compound_prior)
+             / (1.0 - compound_prior)))
     else:
         p_match = 0.35
         p_no_match = max(
             0,
-            ((compound_kegg_not_equal_marg - p_match * compound_prior) /
-             (1.0 - compound_prior)))
+            ((compound_kegg_not_equal_marg - p_match * compound_prior)
+             / (1.0 - compound_prior)))
 
     return p_match, p_no_match
 
@@ -367,97 +387,109 @@ def reaction_id_likelihood(
         r1, r2, reaction_prior,
         reaction_id_equal_marg, reaction_id_not_equal_marg):
     if util.id_equals(r1.id, r2.id):
-        p_match = 0.52
+        p_match = 0.3
         p_no_match = max(
             0,
-            ((reaction_id_equal_marg - p_match * reaction_prior) /
-             (1.0 - reaction_prior)))
+            ((reaction_id_equal_marg - p_match * reaction_prior)
+             / (1.0 - reaction_prior)))
     else:
-        p_match = 0.48
+        p_match = 0.7
         p_no_match = max(
             0,
-            ((reaction_id_not_equal_marg - p_match * reaction_prior) /
-             (1.0 - reaction_prior)))
+            ((reaction_id_not_equal_marg - p_match * reaction_prior)
+             / (1.0 - reaction_prior)))
 
     return p_match, p_no_match
 
 
 def reaction_name_likelihood(r1, r2, reaction_prior, reaction_name_marg):
     if util.name_equals(r1.name, r2.name):
-        p_match = 0.59
+        p_match = 0.2
         p_no_match = max(
             0,
-            ((reaction_name_marg - p_match * reaction_prior) /
-             (1.0 - reaction_prior)))
+            ((reaction_name_marg - p_match * reaction_prior)
+             / (1.0 - reaction_prior)))
     else:
-        p_match = 0.41
+        p_match = 0.8
         p_no_match = max(
             0,
-            ((1.0 - reaction_name_marg - p_match * reaction_prior) /
-             (1.0 - reaction_prior)))
+            ((1.0 - reaction_name_marg - p_match * reaction_prior)
+             / (1.0 - reaction_prior)))
 
     return p_match, p_no_match
 
 
 def reaction_equation_mapping_approx_max_likelihood(
-        cpd_set1, cpd_set2, cpd_pred, compartment_map={}):
+        cpd_set1, cpd_set2, cpd_map, cpd_score, compartment_map={}):
     """Calculate equation likelihood based on compound mapping."""
-    p_match = 0.0
-    p_no_match = 0.0
+    p_match = 1.0
+    p_no_match = 1.0
 
-    # get the possible best-match pairs
-    pair_list = list(set(
-        (c1.name, c2.name)
-        for c1, c2 in product(cpd_set1, cpd_set2)
-        if ((c1.name, c2.name) in cpd_pred.index
-            and (compartment_map.get(c1.compartment, c1.compartment)
-                 == c2.compartment))))
+    compartment_dict = defaultdict(set)
+    for c in cpd_set2:
+        compartment_dict[c.name].add(c.compartment)
 
-    # remove the compartment information
-    cpd_set1 = [c.name for c in cpd_set1]
-    cpd_set2 = [c.name for c in cpd_set2]
+    # get the possible best-match pairs, score as the key
+    best_match = defaultdict(set)
+    for c1 in cpd_set1:
+        for c2 in cpd_map[c1.name].intersection(compartment_dict.keys()):
+            # the compound pair should be in the same compartment
+            if (compartment_map.get(c1.compartment, c1.compartment)
+                    in compartment_dict[c2]):
+                best_match[cpd_score[c1.name]].add((c1.name, c2))
 
-    # get the p value for (c1, c2) pairs, high possibility first
-    cpd_pred = (cpd_pred.loc[pair_list]
-                .sort_values(ascending=False))
+    # translate each element to compound id
+    cpd_set1 = set(c.name for c in cpd_set1)
+    cpd_set2 = set(c.name for c in cpd_set2)
 
-    if (len(cpd_pred) > 0):  # if best-hit pairs exist
-        for c1, c2 in cpd_pred.index.values:
+    # sort by scores, get the most possible compound pairs first
+    for score in sorted(best_match.keys(), reverse=True):
+        for c1, c2 in best_match[score]:
             if (c1 in cpd_set1) and (c2 in cpd_set2):
-                score = cpd_pred[(c1, c2)]
                 # the possibility that compounds are equal
-                p_match += np.log(score * 0.9 + (1 - score) * 0.1)
-                p_no_match += np.log(score * 0.1 + (1 - score) * 0.9)
+                p_match *= score * 0.9 + (1 - score) * 0.1
+                p_no_match *= score * 0.1 + (1 - score) * 0.9
                 cpd_set1.remove(c1)
                 cpd_set2.remove(c2)
+                if len(cpd_set1) == 0 or len(cpd_set2) == 0:
+                    break
 
     for c in cpd_set1:
-        p_match += np.log(0.1)
-        p_no_match += np.log(0.9)
+        p_match *= 0.1
+        p_no_match *= 0.9
     for c in cpd_set2:
-        p_match += np.log(0.1)
-        p_no_match += np.log(0.9)
+        p_match *= 0.1
+        p_no_match *= 0.9
 
-    p_match = np.exp(p_match)
-    p_no_match = np.exp(p_no_match)
     return p_match, p_no_match
 
 
-def reaction_equation_compound_mapping_likelihood(r1, r2, cpd_pred,
-                                                  compartment_map={}):
+def reaction_equation_compound_mapping_likelihood(
+        r1, r2, *args, **kwargs):
+    """Get the likelihood of reaction equations
+
+    Args:
+        r1, r2: two `RactionEntry` objects to be compared
+    args, kwargs:
+        cpd_map: dictionary mapping compound id in the query model to a set
+                 of best-mapping compound ids in the target model.
+        cpd_score: dictionary mapping compound id in the query model to
+                   its best mapping score during compound mapping.
+        compartment_map: dictionary mapping compartment id in the query model
+                         to the id in the target model.
+    """
     if r1.equation is None or r2.equation is None:
         # p value of observing undefined equation
         # it is independent of the condition of match or not
         p_match = 1
         p_no_match = 1
     else:
-        p_match, p_no_match = get_best_p_value_set(r1, r2, cpd_pred,
-                                                   compartment_map)
+        p_match, p_no_match = get_best_p_value_set(r1, r2, *args, **kwargs)
 
     return p_match, p_no_match
 
 
-def get_best_p_value_set(r1, r2, cpd_pred, compartment_map):
+def get_best_p_value_set(r1, r2, *args, **kwargs):
     """Assume equations may have reversed direction, report best mapping p."""
     cpd_set1_left = get_cpd_set(r1.equation, left=True)
     cpd_set1_right = get_cpd_set(r1.equation, left=False)
@@ -466,12 +498,12 @@ def get_best_p_value_set(r1, r2, cpd_pred, compartment_map):
 
     # assume equations have the same direction
     p_forward_match, p_forward_no_match = merge_partial_p_set(
-        cpd_set1_left, cpd_set2_left, cpd_pred,
-        cpd_set1_right, cpd_set2_right, compartment_map)
+        cpd_set1_left, cpd_set2_left,
+        cpd_set1_right, cpd_set2_right, *args, **kwargs)
     # assume equations have the reversed direction
     p_reverse_match, p_reverse_no_match = merge_partial_p_set(
-        cpd_set1_left, cpd_set2_right, cpd_pred,
-        cpd_set1_right, cpd_set2_left, compartment_map)
+        cpd_set1_left, cpd_set2_right,
+        cpd_set1_right, cpd_set2_left, *args, **kwargs)
 
     # maintain the direction with better p values
     if (p_forward_match / p_forward_no_match
@@ -485,8 +517,8 @@ def get_best_p_value_set(r1, r2, cpd_pred, compartment_map):
     return p_match, p_no_match
 
 
-def merge_partial_p_set(cpd_set1_left, cpd_set2_left, cpd_pred,
-                        cpd_set1_right, cpd_set2_right, compartment_map):
+def merge_partial_p_set(cpd_set1_left, cpd_set2_left,
+                        cpd_set1_right, cpd_set2_right, *args, **kwargs):
     """Merge the left hand side and right hand side p values together.
 
     The compound mapping is done separately on left hand side and
@@ -495,10 +527,10 @@ def merge_partial_p_set(cpd_set1_left, cpd_set2_left, cpd_pred,
     """
     p_set_left = \
         reaction_equation_mapping_approx_max_likelihood(
-            cpd_set1_left, cpd_set2_left, cpd_pred, compartment_map)
+            cpd_set1_left, cpd_set2_left, *args, **kwargs)
     p_set_right = \
         reaction_equation_mapping_approx_max_likelihood(
-            cpd_set1_right, cpd_set2_right, cpd_pred, compartment_map)
+            cpd_set1_right, cpd_set2_right, *args, **kwargs)
     p_match = p_set_left[0] * p_set_right[0]
     p_no_match = p_set_left[1] * p_set_right[1]
     return p_match, p_no_match
@@ -517,26 +549,23 @@ def get_cpd_set(equation, left=True):
     return cpd_set
 
 
-def reaction_genes_likelihood(r1, r2):
+def reaction_genes_likelihood(r1, r2, reaction_prior, reaction_genes_marg,
+                              reaction_genes_not_equal_marg, gene_map={}):
     if r1.genes is None or r2.genes is None:
-        # p value of observing undefined genes
-        # it is independent of the condition of match or not
         p_match = 1
         p_no_match = 1
+    elif util.genes_equals(r1.genes, r2.genes, gene_map):
+        p_match = 0.2
+        p_no_match = max(
+            0,
+            ((reaction_genes_marg - p_match * reaction_prior)
+             / (1.0 - reaction_prior)))
     else:
-        # calculating p_match
-        present = len(r1.genes & r2.genes)
-        differ = len(r1.genes ^ r2.genes)
-        # total = len(r1.genes | r2.genes)
-
-        p_present = 0.99
-        p_differ = 0.01
-        p_match = p_present**present * p_differ**differ
-
-        # calculating p_no_match
-        p_present = 0.10
-        p_differ = 0.90
-        p_no_match = p_present**present * p_differ**differ
+        p_match = 0.8
+        p_no_match = max(
+            0,
+            ((reaction_genes_not_equal_marg - p_match * reaction_prior)
+             / (1.0 - reaction_prior)))
 
     return p_match, p_no_match
 
@@ -547,18 +576,19 @@ def fake_likelihood(e1, e2):
 
 
 def generate_likelihood(tasks):
-    pair, likelihood, params = tasks
+    pair, likelihood, args, kwargs = tasks
     e1, e2 = pair
-    p1, p2 = likelihood(e1, e2, *params)
+    p1, p2 = likelihood(e1, e2, *args, **kwargs)
     return e1.id, e2.id, p1, p2
 
 
-def pairwise_likelihood(pool, chunksize, model1, model2, likelihood, params):
+def pairwise_likelihood(pool, chunksize, model1, model2, likelihood,
+                        *args, **kwargs):
     """Compute likelihood of all pairwise comparisons.
 
     Returns likelihoods as a dataframe with a column for each hypothesis.
     """
-    tasks = (((e1, e2), likelihood, params)
+    tasks = (((e1, e2), likelihood, args, kwargs)
              for e1, e2 in product(itervalues(model1), itervalues(model2)))
     result = pool.map(generate_likelihood, tasks, chunksize=chunksize)
     return pd.DataFrame.from_records(result, index=('e1', 'e2'),
@@ -596,62 +626,77 @@ def map_model_compounds(model1, model2, nproc=1, outpath='.',
     # Initialize parallel pool of workers
     chunksize = compound_pairs // nproc
     pool = Pool(nproc)
+
+    t = time.time()
     # Compound ID
+    print('Calculating compound ID likelihoods...', end=' ')
+    sys.stdout.flush()
+
     # Marginal probability of observing two equal compound IDs
     tasks = ((util.id_equals, (c1.id, c2.id)) for c1, c2 in product(
         itervalues(model1.compounds), itervalues(model2.compounds)))
     result = pool.map(parallel_equel, tasks, chunksize=chunksize)
     compound_id_marg = sum(result) / float(compound_pairs)
 
-    print('Calculating compound ID likelihoods...')
-    sys.stdout.flush()
     compound_id_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.compounds, model2.compounds,
-        compound_id_likelihood, (compound_prior, compound_id_marg))
+        compound_id_likelihood, compound_prior, compound_id_marg)
 
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
     # Compound name
+    print('Calculating compound name likelihoods...', end=' ')
+    sys.stdout.flush()
+
     # Marginal probability of observing two similar names
     tasks = ((util.name_equals, (c1.name, c2.name)) for c1, c2 in product(
         itervalues(model1.compounds), itervalues(model2.compounds)))
     result = pool.map(parallel_equel, tasks, chunksize=chunksize)
     compound_name_marg = sum(result) / float(compound_pairs)
 
-    print('Calculating compound name likelihoods...')
-    sys.stdout.flush()
     compound_name_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.compounds, model2.compounds,
-        compound_name_likelihood, (compound_prior, compound_name_marg))
+        compound_name_likelihood, compound_prior, compound_name_marg)
+
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
 
     # Compound charge
+    print('Calculating compound charge likelihoods...', end=' ')
+    sys.stdout.flush()
+
     # Marginal probability of observing two compounds with the same charge
     compound_charge_equal_marg = sum(
-        c1.charge is not None and
-        c2.charge is not None and
-        c1.charge == c2.charge
+        c1.charge is not None
+        and c2.charge is not None
+        and c1.charge == c2.charge
         for c1, c2 in product(
             itervalues(model1.compounds), itervalues(model2.compounds))
     ) / compound_pairs
 
     # Marginal probability of observing two compounds with different charge
     compound_charge_not_equal_marg = sum(
-        c1.charge is not None and
-        c2.charge is not None and
-        c1.charge != c2.charge
+        c1.charge is not None
+        and c2.charge is not None
+        and c1.charge != c2.charge
         for c1, c2 in product(
             itervalues(model1.compounds), itervalues(model2.compounds))
     ) / compound_pairs
 
-    print('Calculating compound charge likelihoods...')
-    sys.stdout.flush()
-
     compound_charge_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.compounds, model2.compounds,
-        compound_charge_likelihood, (
-            compound_prior,
-            compound_charge_equal_marg,
-            compound_charge_not_equal_marg))
+        compound_charge_likelihood,
+        compound_prior,
+        compound_charge_equal_marg,
+        compound_charge_not_equal_marg)
+
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
 
     # Compound formula
+    print('Calculating compound formula likelihoods...', end=' ')
+    sys.stdout.flush()
+
     # Marginal probability of observing two compounds with the same formula
     tasks = ((
         util.formula_equals,
@@ -665,25 +710,29 @@ def map_model_compounds(model1, model2, nproc=1, outpath='.',
     compound_formula_not_equal_marg = 1.0 - compound_formula_equal_marg - (
         sum(c1.formula is None or c2.formula is None
             for c1, c2 in product(itervalues(model1.compounds),
-                                  itervalues(model2.compounds))) /
-        compound_pairs)
+                                  itervalues(model2.compounds)))
+        / compound_pairs)
 
-    print('Calculating compound formula likelihoods...')
-    sys.stdout.flush()
     compound_formula_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.compounds, model2.compounds,
-        compound_formula_likelihood, (
-            compound_prior, compound_formula_equal_marg,
-            compound_formula_not_equal_marg))
+        compound_formula_likelihood,
+        compound_prior, compound_formula_equal_marg,
+        compound_formula_not_equal_marg)
+
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
 
     # Compound KEGG id
     if kegg:  # run KEGG id mapping
+        print('Calculating compound KEGG ID likelihoods...', end=' ')
+        sys.stdout.flush()
+
         # Marginal probability of observing two compounds
         # where KEGG ids are equal
         compound_kegg_equal_marg = sum(
-            c1.kegg is not None and
-            c2.kegg is not None and
-            c1.kegg == c2.kegg
+            c1.kegg is not None
+            and c2.kegg is not None
+            and c1.kegg == c2.kegg
             for c1, c2 in product(
                 itervalues(model1.compounds),
                 itervalues(model2.compounds))
@@ -692,24 +741,25 @@ def map_model_compounds(model1, model2, nproc=1, outpath='.',
         # Marginal probability of observing two compounds
         # where KEGG ids are different
         compound_kegg_not_equal_marg = sum(
-            c1.kegg is not None and
-            c2.kegg is not None and
-            c1.kegg != c2.kegg for c1, c2 in product(
+            c1.kegg is not None
+            and c2.kegg is not None
+            and c1.kegg != c2.kegg for c1, c2 in product(
                 itervalues(model1.compounds),
                 itervalues(model2.compounds))
         ) / compound_pairs
 
-        print('Calculating compound KEGG ID likelihoods...')
-        sys.stdout.flush()
         compound_kegg_likelihoods = pairwise_likelihood(
             pool, chunksize, model1.compounds, model2.compounds,
-            compound_kegg_likelihood, (
-                compound_prior, compound_kegg_equal_marg,
-                compound_kegg_not_equal_marg))
+            compound_kegg_likelihood,
+            compound_prior, compound_kegg_equal_marg,
+            compound_kegg_not_equal_marg)
+
+        print('%.2f seconds' % (time.time() - t))
+        t = time.time()
     else:  # run fake mapping
         compound_kegg_likelihoods = pairwise_likelihood(
             pool, chunksize, model1.compounds, model2.compounds,
-            fake_likelihood, ())
+            fake_likelihood)
 
     pool.close()
     pool.join()
@@ -746,8 +796,9 @@ def map_model_compounds(model1, model2, nproc=1, outpath='.',
             bayes_posterior(compound_prior, compound_kegg_likelihoods))
 
 
-def map_model_reactions(model1, model2, cpd_pred, nproc=1, outpath='.',
-                        log=False, gene=False, compartment_map={}):
+def map_model_reactions(model1, model2, cpd_map, cpd_score, nproc=1,
+                        outpath='.', log=False, gene=False, compartment_map={},
+                        gene_map={}):
     """Map reactions of two models."""
     # Mapping of reactions
     reaction_pairs = len(model1.reactions) * len(model2.reactions)
@@ -763,6 +814,10 @@ def map_model_reactions(model1, model2, cpd_pred, nproc=1, outpath='.',
     pool = Pool(nproc)
 
     # Reaction ID
+    t = time.time()
+    print('Calculating reaction ID likelihoods...', end=' ')
+    sys.stdout.flush()
+
     # Marginal probability of observing two reactions with the same ids.
     tasks = ((util.id_equals, (r1.id, r2.id)) for r1, r2 in product(
         itervalues(model1.reactions),
@@ -773,15 +828,17 @@ def map_model_reactions(model1, model2, cpd_pred, nproc=1, outpath='.',
     # Marginal probability of observing two reactions with different ids.
     reaction_id_not_equal_marg = 1.0 - reaction_id_equal_marg
 
-    print('Calculating reaction ID likelihoods...')
-    sys.stdout.flush()
     reaction_id_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.reactions, model2.reactions,
-        reaction_id_likelihood, (
-            reaction_prior,
-            reaction_id_equal_marg, reaction_id_not_equal_marg))
+        reaction_id_likelihood,
+        reaction_prior, reaction_id_equal_marg, reaction_id_not_equal_marg)
 
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
     # Reaction name
+    print('Calculating reaction name likelihoods...', end=' ')
+    sys.stdout.flush()
+
     # Marginal probability of observing two reactions with the same name.
     tasks = ((util.name_equals, (r1.name, r2.name)) for r1, r2 in product(
         itervalues(model1.reactions),
@@ -789,20 +846,24 @@ def map_model_reactions(model1, model2, cpd_pred, nproc=1, outpath='.',
     result = pool.map(parallel_equel, tasks, chunksize=chunksize)
     reaction_name_equal_marg = sum(result) / float(reaction_pairs)
 
-    print('Calculating reaction name likelihoods...')
-    sys.stdout.flush()
     reaction_name_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.reactions, model2.reactions,
-        reaction_name_likelihood, (reaction_prior, reaction_name_equal_marg))
+        reaction_name_likelihood, reaction_prior, reaction_name_equal_marg)
+
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
 
     # Reaction equation
 
-    print('Calculating reaction equation likelihoods...')
+    print('Calculating reaction equation likelihoods...', end=' ')
     sys.stdout.flush()
     reaction_equation_likelihoods = pairwise_likelihood(
         pool, chunksize, model1.reactions, model2.reactions,
         reaction_equation_compound_mapping_likelihood,
-        (cpd_pred, compartment_map))
+        cpd_map, cpd_score, compartment_map)
+
+    print('%.2f seconds' % (time.time() - t))
+    t = time.time()
 
     # Reaction genes
     # For each gene, the marginal probability of observing that gene
@@ -810,15 +871,38 @@ def map_model_reactions(model1, model2, cpd_pred, nproc=1, outpath='.',
     # observing a pair of genes in two reactions given that the reaction
     # do _not_ match.
     if gene:
-        print('Calculating reaction genes likelihoods...')
+        print('Calculating reaction genes likelihoods...', end=' ')
         sys.stdout.flush()
+
+        # Marginal probability of observing two reactions with
+        # equal gene associations.
+        tasks = ((util.genes_equals, (r1.genes, r2.genes))
+                 for r1, r2 in product(
+                     itervalues(model1.reactions),
+                     itervalues(model2.reactions)))
+        result = pool.map(parallel_equel, tasks, chunksize=chunksize)
+        reaction_genes_equal_marg = sum(result) / float(reaction_pairs)
+
+        # Marginal probability of observing two reactions with unequal
+        # gene associations.
+        reaction_genes_not_equal_marg = 1.0 - reaction_genes_equal_marg - (
+            sum(r1.genes is None or r2.genes is None
+                for r1, r2 in product(itervalues(model1.reactions),
+                                      itervalues(model2.reactions)))
+            / reaction_pairs)
+
         reaction_genes_likelihoods = pairwise_likelihood(
             pool, chunksize, model1.reactions, model2.reactions,
-            reaction_genes_likelihood, ())
+            reaction_genes_likelihood,
+            reaction_prior, reaction_genes_equal_marg,
+            reaction_genes_not_equal_marg, gene_map)
+
+        print('%.2f seconds' % (time.time() - t))
+        t = time.time()
     else:
         reaction_genes_likelihoods = pairwise_likelihood(
             pool, chunksize, model1.reactions, model2.reactions,
-            fake_likelihood, ())
+            fake_likelihood)
 
     pool.close()
     pool.join()
@@ -848,3 +932,17 @@ def map_model_reactions(model1, model2, cpd_pred, nproc=1, outpath='.',
             bayes_posterior(reaction_prior, reaction_name_likelihoods),
             bayes_posterior(reaction_prior, reaction_equation_likelihoods),
             bayes_posterior(reaction_prior, reaction_genes_likelihoods))
+
+
+def check_cpd_charge(compound, source):
+    if compound.charge is not None:
+        try:
+            int(compound.charge)
+            return 'TRUE'
+        except ValueError:
+            logger.warning(
+                "Compound charge should be an integer, however, charge of "
+                "compound {} in {} is '{}', which is invalid. Please remove "
+                "or fix it before running modelmapping command.".format(
+                    compound.id, source, compound.charge))
+            return 'FALSE'
