@@ -107,6 +107,15 @@ class ModelMappingCommand(Command):
                 'if the compartment ids in the two models are not consistent.'
             )
         )
+        parser_mm.add_argument(
+            '--gene-map-file', type=str,
+            help=(
+                'A tab delimited file (.tsv) listing the gene ids of '
+                'query model in the first column, and the corresponding '
+                'gene ids of dest model in the second column. Required '
+                'if the gene ids in the two models are not consistent.'
+            )
+        )
 
         # manual curation subcommand
         parser_c = subparsers.add_parser(
@@ -172,6 +181,27 @@ class ModelMappingCommand(Command):
 
     def run(self):
         """Run model mapping command."""
+
+        # Parse models
+        model_1 = ModelReader.reader_from_path(self._args.model).create_model()
+        model_2 = ModelReader.reader_from_path(
+            self._args.dest_model).create_model()
+
+        # check if all compound charge can be converted to an integer
+        invalid_cpd = []
+        for compound in model_1.compounds:
+            result = bayesian.check_cpd_charge(compound, 'your model')
+            if result == 'FALSE':
+                invalid_cpd.append(compound.id)
+
+        for compound in model_2.compounds:
+            result = bayesian.check_cpd_charge(compound, 'dest model')
+            if result == 'FALSE':
+                invalid_cpd.append(compound.id)
+
+        if len(invalid_cpd) != 0:
+            quit()
+
         which_command = self._args.which
         if which_command == 'mm':
             self._model_mapping()
@@ -217,12 +247,17 @@ class ModelMappingCommand(Command):
                     source, target = row.strip().split()
                     compartment_map[source] = target
 
+        # Load gene mapping
+        gene_map = {}
+        if self._args.gene_map_file is not None:
+            with open(self._args.gene_map_file, 'r') as f:
+                for row in f:
+                    source, target = row.strip().split()
+                    gene_map[source] = target
+
         # Check models
-        if (
-            not (
-                model1.check_reaction_compounds() and
-                model2.check_reaction_compounds()) and
-                self._args.consistency_check):
+        if (not (model1.check_reaction_compounds()
+                 and model2.check_reaction_compounds())):
             quit((
                 '\nError: '
                 'equations have something not listed in compounds.yaml, '
@@ -230,7 +265,7 @@ class ModelMappingCommand(Command):
 
         mkdir_p(self._args.outpath)
 
-        if (actual_compound_mapping is not None):
+        if actual_compound_mapping is not None:
             mkdir_p(self._args.outpath + '/roc')
 
         # Bayesian classifier
@@ -240,16 +275,16 @@ class ModelMappingCommand(Command):
             model1, model2, self._args.nproc, self._args.outpath,
             log=self._args.log, kegg=self._args.map_compound_kegg)
         print(
-            'It took %s seconds to calculate compound mapping...'
+            'It took %.2f seconds to calculate compound mapping...'
             % (time.time() - t))
 
         print('Writing output...')
         sys.stdout.flush()
         t = time.time()
         # Write out ROC curve results
-        if (actual_compound_mapping is not None):
-            with open(self._args.outpath +
-                      '/roc/compound_bayes.tsv', 'w') as f:
+        if actual_compound_mapping is not None:
+            with open(self._args.outpath
+                      + '/roc/compound_bayes.tsv', 'w') as f:
                 write_roc_curve(f, cpd_bayes_pred.model1.compounds,
                                 cpd_bayes_pred.model2.compounds,
                                 cpd_bayes_pred,
@@ -258,14 +293,16 @@ class ModelMappingCommand(Command):
         # Parse and output raw mapping
         if self._args.raw:
             cpd_bayes_pred.get_raw_map().to_csv(
-                self._args.outpath + '/bayes_compounds.tsv', sep='\t')
+                self._args.outpath + '/bayes_compounds.tsv', sep='\t',
+                encoding='utf-8')
 
         # Output best mapping
         compound_best = cpd_bayes_pred.get_best_map(
             self._args.threshold_compound)
         compound_best.to_csv(
-            self._args.outpath + '/bayes_compounds_best.tsv', sep='\t')
-        print('It took %s seconds to write output...' % (time.time() - t))
+            self._args.outpath + '/bayes_compounds_best.tsv', sep='\t',
+            encoding='utf-8')
+        print('It took %.2f seconds to write output...' % (time.time() - t))
         sys.stdout.flush()
 
         # Bayesian classifier
@@ -274,18 +311,18 @@ class ModelMappingCommand(Command):
             model1, model2, compound_best.loc[:, 'p'],
             self._args.nproc, self._args.outpath,
             log=self._args.log, gene=self._args.map_reaction_gene,
-            compartment_map=compartment_map)
+            compartment_map=compartment_map, gene_map=gene_map)
         print(
-            'It took %s seconds to calculate reaction mapping...'
+            'It took %.2f seconds to calculate reaction mapping...'
             % (time.time() - t))
 
         print('Writing output...')
         sys.stdout.flush()
         t = time.time()
         # Write out ROC curve results
-        if (actual_reaction_mapping is not None):
-            with open(self._args.outpath +
-                      '/roc/reaction_bayes.tsv', 'w') as f:
+        if actual_reaction_mapping is not None:
+            with open(self._args.outpath
+                      + '/roc/reaction_bayes.tsv', 'w') as f:
                 write_roc_curve(f, rxn_bayes_pred.model1.reactions,
                                 rxn_bayes_pred.model2.reactions,
                                 rxn_bayes_pred,
@@ -294,14 +331,16 @@ class ModelMappingCommand(Command):
         # Parse and output raw mapping
         if self._args.raw:
             rxn_bayes_pred.get_raw_map().to_csv(
-                self._args.outpath + '/bayes_reactions.tsv', sep='\t')
+                self._args.outpath + '/bayes_reactions.tsv', sep='\t',
+                encoding='utf-8')
 
         # Output best mapping
         reaction_best = rxn_bayes_pred.get_best_map(
             self._args.threshold_reaction)
         reaction_best.to_csv(
-            self._args.outpath + '/bayes_reactions_best.tsv', sep='\t')
-        print('It took %s seconds to write output...' % (time.time() - t))
+            self._args.outpath + '/bayes_reactions_best.tsv', sep='\t',
+            encoding='utf-8')
+        print('It took %.2f seconds to write output...' % (time.time() - t))
         sys.stdout.flush()
 
     def _curation(self):
@@ -316,7 +355,7 @@ class ModelMappingCommand(Command):
             self._args.compound_map, self._args.reaction_map,
             self._args.curated_compound_map, self._args.curated_reaction_map)
 
-        # read commpartment map
+        # read compartment map
         compartment_map = {}
         if self._args.compartment_map is not None:
             with open(self._args.compartment_map) as f:
