@@ -443,7 +443,9 @@ def count_genes(model):
         if isinstance(reaction.genes, boolean.Expression):
             genes.update(v.symbol for v in reaction.genes.variables)
         else:
-            genes.update(reaction.genes)
+            gene_exp = boolean.Expression(reaction.genes)
+            genes.update(v.symbol for v in gene_exp.variables)
+            # genes.update(reaction.genes)
 
     return len(genes)
 
